--[[
	Total Rewrite:
	---------------------------------------
	To Do List for 1.0:
	* Clean up the age and accuracy in the tooltip
	* Add color based score for over 100% accuracy
	* Add color for age, 1 for < SIL_CacheAge, degrading down to 0.2 over a day
	* Add color to the XXX% section of the tooltip
	* Support for party and raid scanning
	
	Long Term To-Do List:
	* Add SiL's AiL to your charciter information window
	* Add the AiL to the inspect window
	* UI for viwing the cache
	* More mathmatical stats for party and raid
	
	Known Issues:
	* Still getting dupe tooltips
	* Blizzard_InspectUI\InspectPaperDollFrame.lua errors are bugs in the default UI
	
	Changelog for 0.3:
	* Total rewrite
	* Ignores heirloom's since they dont have a iLevel
	* Changed terminology to Average iLevel, AiL to fix the vernacular
	* Added internal methods of looking up people by name and guid
	* Added age and accuracy % to the tooltip, /sil advanced to toggle
	* Lot more documentation inside the code helping keep me sane
	* Now displays AiL at 1 decimal place
	* Due to big changes in the handling of cache it will be reset
	
]]

-- Variables
SIL_Loaded = false;
SIL_Debug = false;
SIL_Version = 0.3
SIL_CurrentTooltip = '';



function SIL_OnEvent(SIL_Nil, eventName, arg1, arg2, arg3)
	if ( eventName == 'ADDON_LOADED' ) and not ( SIL_Loaded ) then
		
		-- Set saved variables
		if not ( SIL_CacheGUID ) then
			SIL_Initialize();
		end
		
		-- Tell the player we have loaded
		SIL_Console('Loading v'..SIL_Version);
		SIL_Loaded = true;
		
		-- Debug if its the devs alt
		if ( UnitGUID('player') == '0x058000000657392B' ) then
			SIL_Debug = true;
		end
	end
	
	if ( eventName == 'PLAYER_TARGET_CHANGED' ) then
		SIL_StartScore('target', false, true);
	end
	
	if ( eventName == 'INSPECT_READY' ) then	
		SIL_ProcessInspect(arg1); 
	end
	
	-- This was a test that failed
	if ( eventName == 'UPDATE_MOUSEOVER_UNIT' ) and not ( InCombatLockdown() ) and ( CanInspect("mouseover") ) then
		SIL_CurrentTooltip = '';
	end
end;

--[[
	SIL_Initialize();
	Clears settings/resets on inital load
]]
function SIL_Initialize()
	SIL_CacheGUID = {}; -- [UnitGUID] = {'items'=>#,'total'=>#,'name'=>UnitName,'time'=>time()};
	SIL_CacheName = {}; -- [UnitName] = UnitGUID;
	SIL_CacheAge = 1800; -- How old till information is refreshed
	SIL_Accuracy = 14; -- How many items is considered 100% accurate
	SIL_Advanced = false; -- Display extra information in the tooltips
end;

--[[
	SIL_HasScore(UnitName|UnitGUID, name|guid);
	returns true or false
]]
function SIL_HasScore(target, targetType)
	targetType = strlower(targetType);
	
	-- Try and convert the name to a GUID
	if ( targetType == 'name' ) and ( SIL_CacheName[target] ) then
		targetType = 'guid';
		target = SIL_CacheName[target];
	end
	
	if ( targetType == "guid" ) and ( SIL_CacheGUID[target] ) and ( SIL_CacheGUID[target]['items'] ) then
		return true;
	else
		return false;
	end
end

--[[
	SIL_GetScore(UnitName|UnitGUID, name|guid);
	returns false or score, accuracy, age
]]
function SIL_GetScore(target, targetType)
	targetType = strlower(targetType);
	
	-- Try and convert the name to a GUID
	if ( targetType == 'name' ) and ( SIL_CacheName[target] ) then
		targetType = 'guid';
		target = SIL_CacheName[target];
	end
	
	-- If we still have a name then this failed
	if ( targetType == 'name' ) then
		return false;
		
	-- We have a GUID and some information to return
	elseif ( targetType == "guid" ) and ( SIL_CacheGUID[target] ) and ( SIL_CacheGUID[target]['items'] ) then
		
		-- Return the unformated score and accuracy
		local score = SIL_CacheGUID[target]['total'] / SIL_CacheGUID[target]['items'];
		local accuracy = SIL_GetAccuracy(SIL_CacheGUID[target]['items'], true);
		local age = time() - SIL_CacheGUID[target]['time']
		return score, accuracy, age;
	else
		return false;
	end
end

--[[
	SIL_SetScore(items, total, name, guid);
	return true or false
]]
function SIL_SetScore(items, total, name, guid)
	if ( items ) and ( total ) and ( name ) and ( guid ) then
		SIL_CacheGUID[guid]['items'] = items;
		SIL_CacheGUID[guid]['total'] = total;
		SIL_CacheGUID[guid]['name'] = name;
		SIL_CacheGUID[guid]['time'] = time();
		
		SIL_CacheName[name] = guid;
		
		return true;
	else
		return false;
	end
end;

--[[
	SIL_ClearScore(UnitName|UnitGUID, name|guid);
	return true or false
]]
function SIL_ClearScore(target, targetType)
	targetType = strlower(targetType);
	
	-- Try and convert the name to a GUID
	if ( targetType == 'name' ) and ( SIL_CacheName[target] ) then
		targetType = 'guid';
		target = SIL_CacheName[target];
	end
	
	-- If we still have a name then this failed
	if ( targetType == 'name' ) then
		return false;
		
	-- We have a GUID so clear it
	elseif ( targetType == "guid" ) and ( SIL_CacheGUID[target] ) and ( SIL_CacheGUID[target]['items'] )then
		SIL_CacheGUID[target] = {};
		
		return true;
	else
		return false;
	end
end;

--[[
	SIL_GetAccuracy(items, timesOneHundred)
	if timesOneHundred = true then the score will be 0-100
	returns the accuracy for a give number of items
]]
function SIL_GetAccuracy(items, times)
	local accuracy = 0;
	
	-- Find the accuracy
	if ( items < SIL_Accuracy ) then
		accuracy = items / SIL_Accuracy;
	else
		accuracy = 1;
	end
	
	-- Return a result
	if ( times ) then
		return SIL_Round(accuracy * 100, 1);
	else
		return accuracy;
	end;
end;

--[[
	SIL_StartScore(target, forceRefresh, showTooltip);
	returns true or false if a score was started
]]
function SIL_StartScore(target, refresh, tooltip)
	
	-- Incombat so we can't do anything
	if ( InCombatLockdown() ) then 
		return false;
	
	-- We can inspect the person
	elseif ( CanInspect(target) ) then
		local guid = UnitGUID(target);
		local name = UnitName(target);
		
		-- We want to start from scratch
		if ( refresh ) then
			SIL_Console("Clearing score for "..name, true);
			SIL_ClearScore(guid, 'guid');
			
		end
		
		-- Check there score to get all usefull information
		local score, accuracy, age = SIL_GetScore(guid, 'guid');
		
		-- Some debuging
		if ( score ) then
			SIL_Console(name.." score:"..score.." acc:"..accuracy.." age:"..age, true);
		end
		
		-- We have a score and its under age and its accurate
		if ( score ) and ( age < SIL_CacheAge ) and ( accuracy == '100.0' ) then
			
			if ( tooltip ) then
				SIL_ShowTooltip();
			end
			
			---SIL_Console(name.." Cached "..age, true);
			return false;
		
		-- We can do something!
		else
			---SIL_Console(name.." Fresh", true);
			
			-- Start some some information if we haven't seen them before
			if not ( score ) then
				SIL_CacheName[name] = guid;
				SIL_CacheGUID[guid] = {};
				SIL_CacheGUID[guid]['name'] = name;
			end
			
			-- Update the target information for this person
			SIL_CacheGUID[guid]['target'] = target;
			SIL_CacheGUID[guid]['tooltip'] = tooltip;
			
			-- Start the inspect
			NotifyInspect(target);
			
			return true;
		end
	else
		return false;
	end
end


--[[
	SIL_ProcessInspect(guid, showTooltip)
	returns false or the score and accuracy in %;
]]
function SIL_ProcessInspect(guid, tooltip)
	
	-- Incombat so we can't do anything
	if ( InCombatLockdown() ) then 
		return false;
	
	else
		
		-- We have some more information about this person
		if ( SIL_CacheGUID[guid] ) then
			local name = SIL_CacheGUID[guid]['name'];
			local target = SIL_CacheGUID[guid]['target'];
			
			--- SIL_Console('Processing '..name..' by '..target, true);
			
			-- Figure out wether or not to display the tooltip
			if not ( tooltip ) and not ( tooltip == false ) then
				tooltip = SIL_CacheGUID[guid]['tooltip'];
			end
			
			local totalItems = 0;
			local totalScore = 0;
			
			-- Loop all items
			for i = 1, 18 do
				
				-- Skip the Shirt
				if ( i ~= 4 ) then
					local itemLink = GetInventoryItemLink(target, i);
					
					-- We have a item link
					if ( itemLink ) then
						local _, _, itemRarity , itemLevel = GetItemInfo(itemLink);
						
						-- We have a valid itemLevel and its above white 1, and below artificat 6
						if ( itemLevel ) and ( 1 < itemRarity ) and ( itemRarity < 6) then
							totalItems = totalItems + 1;
							totalScore = totalScore + itemLevel;
						end
					end
				end
			end
			
			-- We have some items to give a score for!
			if ( totalItems > 0 ) then
				
				-- Set there score
				SIL_SetScore(totalItems, totalScore, name, guid);
				
				-- Get the score back, this is dumb but it avoids dupe code
				local score, accuracy = SIL_GetScore(guid, 'guid');
				
				if ( tooltip ) then
					SIL_ShowTooltip(score, totalItems);
				end
				
				--- SIL_Console(name..' has '..totalItems..' items with a score or '..score..' and accuracy of '..accuracy..'%', true);
				return score, accuracy;
			else
				--- SIL_Console(name..' has no items', true);
				return false;
			end
		else
			return false;
		end
	end
end

--[[
	SIL_ShowTooltip();
	Updates the tooltip information
]]
function SIL_ShowTooltip()
	local guid = UnitGUID("mouseover");
	
	if ( SIL_HasScore(guid, 'guid') ) then
	-- if ( SIL_HasScore(guid, 'guid') ) and ( SIL_CurrentTooltip ~= guid ) then
				
		local items = SIL_CacheGUID[guid]['items'];
		local score, accuracyPercent, age = SIL_GetScore(guid, 'guid');
		local accuracy = SIL_GetAccuracy(items, false);
		
		-- Default to white
		rgb = 1;
		
		-- If we are short then change it to gray, no less then 0.2 tho
		if ( 0.2 > accuracy ) and ( accuracy < 1 ) then
			rgb = SIL_Round(accuracy, 2);
		elseif ( accuracy < 0.21 ) then
			rgb = 0.2;
		end
		
		GameTooltip:AddDoubleLine('Average iLevel: ', SIL_Round(score, 1), 0.13,0.42,1, rgb,rgb,rgb);
		GameTooltip:Show();
		
		if ( SIL_Advanced ) then
			GameTooltip:AddDoubleLine(accuracyPercent..'% accuracy', SIL_AgeToText(age), rgb,rgb,rgb);
			GameTooltip:Show();
		end
		
		-- Log it so we dont have a dupe
		--SIL_CurrentTooltip = guid;
		
		return true;
	else
		return false;
	end
end;

--[[
	SIL_TooltipHook();
	Called when you mouse over someone, should attempt to also do mouseover event
	return true or false if a tooltip was able to be shown
]]
function SIL_TooltipHook(arg1, arg2)
	
	-- Can't do anything if we are incombat
	if ( InCombatLockdown() ) then 
		return false; 
	else
		return SIL_ShowTooltip();
	end;
end;

--[[
	SIL_AgeToText(age);
]]
function SIL_AgeToText(age)
	if ( age > 86400 ) then
		return SIL_Round(age / 86400, 2)..' days';
	elseif ( age > 3600 ) then
		return SIL_Round(age / 3600, 1)..' hours';
	elseif ( age > 60 ) then
		return SIL_Round(age / 60, 1)..' minutes';
	else
		return age..' seconds';
	end
end


--[[
	SIL_Party(outputInfo)
	return false or partyAverage, partyTotal, partySize
]]
function SIL_Party(output)
	return false;
end

--[[
	SIL_Raid(outputInfo);
	returns false or raidAverage, raidTotal, raidSize
]]
function SIL_Raid(output)
	
	if ( UnitInRaid("player") ) then
		local raidSize = 0;
		local raidTotal = 0;
		local raidAverage = 0;
		
		for i = 1, 40 do
			name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(i);
			
			if ( name ) then
				
				local guid = UnitGUID(name);
				
				-- Start Inspecting
				if ( CanInspect(name) ) then
					SIL_StartScore(name, true, false);
					score, accuracy, age = SIL_ProcessInspect(guid, false);
				
				-- We couldn't inspect so try from the cache
				elseif ( SIL_HasScore(guid, 'guid') ) then
					score, accuracy, age = SIL_GetScore(guid, 'guid');
				end
				
				-- They have a score so count them
				if ( score ) then
					raidSize = raidSize + 1;
					raidTotal = raidTotal + SIL_CacheGUID[guid]['total'];
					
					SIL_Console(name.." AiL:"..score.." "..accuracy.."% "..SIL_AgeToText(age).." old", output);
				end
			end
		end
		
		if ( raidSize > 0 ) then
			local raidAverage = raidTotal / raidSize;
			SIL_Console("------------------------", output);
			SIL_Console("Raid AiL: "..SIL_Round(raidAverage, 1).." "..raidSize.." members", output);
			
			return raidAverage, raidTotal, raidSize;
		else 
			return false;
		end
	else
		return false;
	end
end


--[[
	SIL_SlashCommand(command);
	handleing of /sil
]]
function SIL_SlashCommand(command)
	
	-- Break up the command
	local command, rest = command:match("^(%S*)%s*(.-)$");
	command = strlower(command);
	
	-- clear all the settings
	if ( command == "clear" ) then
		SIL_Console("Clearing settings");
		SIL_Initialize();
	
	-- Toggle debug mode
	elseif ( command == "debug" ) then
		if ( SIL_Debug ) then
			SIL_Console("Debug Off");
			SIL_Debug = false;
		else
			SIL_Console("Debug On");
			SIL_Debug = true;
		end

	-- Toggle debug mode
	elseif ( command == "advanced" ) then
		if ( SIL_Advanced ) then
			SIL_Console("Advanced tooltips Off");
			SIL_Advanced = false;
		else
			SIL_Console("Advanced tooltips On");
			SIL_Advanced = true;
		end
	
	-- Get the AiL of your target
	elseif ( command == "get" ) and not ( rest ) then
		
		-- Make sure we can inspect
		if ( CanInspect('target') ) then
		
			SIL_StartScore('target', true, false);
			local score, accuracy = SIL_ProcessInspect(UnitGUID('target'), false);
			
			if ( score ) then
				SIL_Console(UnitName('target').." has a AiL of "..score.." with "..accuracy.."% accuracy");
			else
				SIL_Console("Sorry, there was a error building a score for your target");
			end
		else
			SIL_Console("Sorry, there was a error inspecting your target");
		end
	
	-- Get the AiL of someone by name
	elseif ( command == "get" ) and ( rest ) then
		
		SIL_Console("Trying to get a score for "..rest, true);
		
		-- Check that we have a score
		if ( SIL_HasScore(rest, 'name') ) then
			
			local score, accuracy, age = SIL_GetScore(rest, 'name');
			
			if ( score ) then
				
				-- Make the age legable
				age = SIL_AgeToText(age);
				
				SIL_Console(rest.." has a AiL of "..score.." with "..accuracy.."% accuracy and the information is "..age.." old");
				
			-- This shouldn't happen
			else
				SIL_Console("Error 3456: HasScore is true but GetScore is false!", true);
				SIL_Console("Error 3456: HasScore is true but GetScore is false!", true);
				SIL_Console("Error 3456: HasScore is true but GetScore is false!", true);
				SIL_Console("Sorry, there was a error getting a score for "..rest);
			end
		else
			SIL_Console("Sorry, there was a error getting a score for "..rest);
		end
	
	-- Set the accuracy
	elseif ( command == "accuracy" ) and ( rest ) and ( type(rest) == "number" ) and ( 0 > rest ) and ( rest < 19 ) then
		SIL_Accuracy = rest;
		SIL_Console("Setting the Accuracy to "..SIL_Accuracy);
	
	elseif (( command == "cache" ) or ( command == "age" )) and ( rest ) and ( type(rest) == "number" ) then
		SIL_CacheAge = rest;
		SIL_Console("Setting the Cache Age to "..SIL_Accuracy);
	
	
	-- Display party information
	elseif ( command == "party" ) then
		SIL_Party(true);
		
	-- Display raid information
	elseif ( command == "raid" ) then
		SIL_Raid(true);
		
	
	-- Display the help
	else
		
		-- Get the score for the player
		SIL_StartScore('player', true, false);
		local score, accuracy = SIL_ProcessInspect(UnitGUID('player'), false);
		
		-- Display help
		SIL_Console("/sil help: for this message");
		SIL_Console("/sil clear: cleared all settings and the cache");
		SIL_Console("/sil advanced: togles advanced tooltips inc age and accuracy");
		SIL_Console("/sil get: Gets the AiL or your current target");
		SIL_Console("/sil get <name>: Gets the AiL of name if its cached");
--		SIL_Console("/sil accuracy <1-18>: Sets the number of items be considered 100%");
--		SIL_Console("/sil age <seconds>: Sets the amount of time between inspect refreshes");
--		SIL_Console("-- Under Devlopment, most likely won't work --");
--		SIL_Console("/sil party: Shows the AiL of everyone in your party that are in range");
--		SIL_Console("/sil raid: Shows the AiL of everyone in your raid that are in range");
		
		if ( score ) then
			SIL_Console('-----------------------');
			SIL_Console('Your AiL is '..score);
		end
	end
end

--[[
	SIL_Round(number, decimals)
	from http://www.wowpedia.org/Round
	returns a rounded number
]]
function SIL_Round(number, decimals)
    return (("%%.%df"):format(decimals)):format(number)
end

--[[
	SIL_Console(message, debugMode);
	ouputs a message to the console, if debugMode=true and SIL_Debug outputs the debug info
]]
function SIL_Console(message, debugMode) 
	if ( SIL_Debug ) and ( debugMode ) then
		print("|cFFbbbbbbDebug:|r "..message);
	elseif not ( debugMode ) then
		print("|cFF216bffSiL:|r "..message);
	end;
end;

-- Create the frame and register the events
local f = CreateFrame("Frame", "SimpleItemLevel", UIParent);
f:SetScript("OnEvent", SIL_OnEvent);
f:RegisterEvent("ADDON_LOADED");
f:RegisterEvent("INSPECT_READY");
f:RegisterEvent("PLAYER_TARGET_CHANGED")
--f:RegisterEvent("UPDATE_MOUSEOVER_UNIT")

-- Hook the tooltip for units
GameTooltip:HookScript("OnTooltipSetUnit", SIL_TooltipHook);

-- Set up the slash command information
SlashCmdList["SIMPLEILEVEL"] = SIL_SlashCommand
SLASH_SIMPLEILEVEL1 = "/sil"
SLASH_SIMPLEILEVEL3 = "/silvl"
SLASH_SIMPLEILEVEL2 = "/simpleitemlevel"